unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics, IOUtils, Math,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ImgList, VirtualTrees, ShlObj, ShellAPI, JPEG, PNGImage,
  WinAPI.GDIPObj, GDIPApi, GDIPUtil, WinAPI.ActiveX, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ToolWin,
  htmlvtree, htutils, htmlcomp, htrtf, htoffice, htmlpars, htcanvas, htcanvasdx, htcanvasgdip, htimage, htxml,
  htpreviewframe, htfonts, htmathml, htcanvastext, htreports, htcanvasgdi, sqlparse,  htofficemail, htscriptparse,
  System.ImageList;

type
  TMainForm = class(TForm)
    FileTree: THtVirtualXMLTree;
    Splitter1: TSplitter;
    ExtImages: TImageList;
    Panel1: TPanel;
    ToolBar1: TToolBar;
    ViewInBrowserBtn: TToolButton;
    HtPanel1: THtPanel;
    PrintBtn: TToolButton;
    ToolButton1: TToolButton;
    SaveDialog1: TSaveDialog;
    SaveDialog2: TSaveDialog;
    LayoutMode: TComboBox;
    procedure HtVirtualXMLTree1Expanding(Sender: TBaseVirtualTree;
      Node: PVirtualNode; var Allowed: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FileTreeFocusChanged(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex);
    procedure FormDestroy(Sender: TObject);
    procedure ViewInBrowserBtnClick(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    procedure LayoutModeChange(Sender: TObject);
  private
    ExtList: TStringList;
    LastFileType: string;
    FileName: string;
    LastDoc: string;
    function AddExtIcon(const Ext: string; Flags: cardinal): integer;
    function RequestPass(Sender: TObject; var Password: hstring): boolean;
  public
    { Public declarations }
  end;


var
  MainForm: TMainForm;

implementation

{$R *.dfm}

function GetSpecialFolderPath(CSIDLFolder: Integer): string;
var
  FilePath: array [0..MAX_PATH] of char;
begin
  SHGetFolderPath(0, CSIDLFolder, 0, 0, FilePath);
  Result := FilePath;
end;

function TMainForm.AddExtIcon(const Ext: string; Flags: cardinal): integer;
var SI: TSHFileInfo;
    IFlags: Cardinal;
    IC: TIcon;
    B: TBitmap;
begin
 IFlags := SHGFI_ICON or SHGFI_SMALLICON or SHGFI_USEFILEATTRIBUTES or SHGFI_TYPENAME;
 SHGetFileInfo(pchar(Ext), Flags, SI, SizeOf(TSHFileInfo), IFlags);
 LastFileType := SI.szTypeName;
 if SI.hIcon > 0 then
 begin
   IC := TIcon.Create;
   try
     IC.Handle := SI.HIcon;
     B := TBitmap.Create;
     try
       B.Width := 16;
       B.Height := 16;
       B.PixelFormat := pf24Bit;
       B.Canvas.Draw(0, 0, Ic);
       Result := ExtImages.AddMasked(B, B.Canvas.Pixels[0, 0]);
     finally
       B.Free;
     end;
     IC.ReleaseHandle;
     finally
       IC.Free;
     end;
 end else
   Result := -1;
end;

type
  TStringRec = class
  public
    s: string;
    constructor Create(const Value: string);
  end;

function MakePagedLayout(const HTML: string): string;
begin
  Result := THtDocumentConverter.ImageConverter.HTMLtoSVG(HTML,
    '.docbody, .part {background:white; margin: 10px 16px; display: inline-block} .docbody a, .part a {text-decoration:inherit;color:inherit}', 0, 0, true);
end;

procedure TMainForm.FileTreeFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
var Ext: string;
    C: IHtDocumentConverter;
    s: string;
    FileData: TBytes;
begin
  if Assigned(Node) and (FileTree.XML(Node)['FILETYPE'] <> 'DIRECTORY') and (FileTree.XML(Node)['FILETYPE'] <> 'DRIVE') then
  begin
    FileName := IncludeTrailingPathDelimiter(FileTree.XML(Node)['PATH']) + FileTree.XML(Node)['NAME'];
    Ext := ExtractFileExt(FileTree.XML(Node)['NAME']);
    if Ext = '' then
    begin
      Ext := '.' + THtDocumentConverter.GetFileFormat(IncludeTrailingPathDelimiter(FileTree.XML(Node)['PATH']) + FileTree.XML(Node)['NAME']);
      if Ext = '.' then
        exit;
    end;
    if StrIn(AnsiUpperCase(Ext), ['.WMF', '.EMF']) then
    begin
      FileData := TFile.ReadAllBytes(IncludeTrailingPathDelimiter(FileTree.XML(Node)['PATH']) + FileTree.XML(Node)['NAME']);
      HtPanel1.HTML.Text := THtEMFImage.ConvertEMF(FileData, Rect(0, 0, 1024, 768));
    end else
    if StrIn(AnsiUpperCase(Ext), ['.SVG', '.HTM']) then
    begin
      HtPanel1.LoadFromFile(IncludeTrailingPathDelimiter(FileTree.XML(Node)['PATH']) + FileTree.XML(Node)['NAME'])
    end else
    begin
      C := THtDocumentConverter.GetConverter(copy(Ext, 2, MaxInt));
      if C = nil then
      begin
        HtPanel1.HTML.Clear;
        exit
      end;
      Screen.Cursor := crAppStart;
      try
        s := C.ConvertFile(IncludeTrailingPathDelimiter(FileTree.XML(Node)['PATH']) + FileTree.XML(Node)['NAME'], RequestPass);
        HtPanel1.HTML.Clear;
        if StrIn(AnsiUpperCase(Ext), ['.PDF', '.PPT', '.PPTX', '.XLS', '.XLSX', '.XLSB', '.MD','.MSG', '.EML']) then
        begin
          if StrIn(AnsiUpperCase(Ext), ['.PDF', '.PPT', '.PPTX']) then
            LastDoc := '<style>body {background: #909095}</style>' + s
          else
            LastDoc := '<style>@media screen { body {background: white; margin: 10px}} @media print {body{ margin: 0 0}}</style>' + s;
        end else
        begin
          if LayoutMode.ItemIndex = 0 then
            LastDoc := '<style>@media screen { body {background: #909095; margin: 10px}} @media print {body{ margin: 0 0}}</style>' + MakePagedLayout(s)
          else
            LastDoc := '<style>@media screen { body {background: white; margin: 10px}} @media print {body{ margin: 0 0}}</style>' + s;
        end;
        HtPanel1.HTML.Text := LastDoc;
      finally
        Screen.Cursor := crDefault;
      end;
    end;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var T: boolean;
begin
  ExtList := TStringList.Create;
  AddExtIcon('', FILE_ATTRIBUTE_DIRECTORY);
  AddExtIcon('', FILE_ATTRIBUTE_NORMAL);
  ExtList.Add('_DRIVE');
  ExtList.Add('_CD');
  ExtList.Add('_DESKTOP');
  ExtList.Add('_FOLDER');
  ExtList.Add('_FILE');
  HtVirtualXMLTree1Expanding(FileTree, nil, T);
  HtPanel1.ClearHTMLAfterLoad := true;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
var i: integer;
begin
  for i := 0 to ExtList.Count - 1 do
    ExtList.Objects[i].Free;
  ExtList.Free;
end;

procedure TMainForm.HtVirtualXMLTree1Expanding(Sender: TBaseVirtualTree;
  Node: PVirtualNode; var Allowed: Boolean);
var Path, Ext: string;
   F: TSearchRec;
   Rows, Row: THtXMLNode;
   t, n: integer;
   vDrives	: array[0..128] of Char;
   vDrive: pchar;
begin
  if Node = nil then
  begin
    Rows := THtXMLNode.Create('<directory/>');
    if GetLogicalDriveStrings(SizeOf(vDrives), vDrives) = 0 then
      exit;
    vDrive := vDrives;
   	while vDrive^ <> #0 do
  	begin
      Row := Rows.Add('R');
      Row['NAME'] := StrPas(vDrive);
	    Inc(vDrive, 4);
      Row['CHILDCOUNT'] := '1';
      Row['FILETYPE']:= 'DRIVE';
      Row['IMAGEINDEX']:= '0';
  	end;
    FileTree.LoadfromXML(Rows, Node);
    exit;
  end;
  if FileTree.XML(Node)['FILETYPE'] <> 'DRIVE' then
    Path := IncludeTrailingPathDelimiter(FileTree.XML(Node)['PATH']) + FileTree.XML(Node)['NAME'] + '\*.*'
  else
    path := FileTree.XML(Node)['NAME'] + '*.*';
  if FindFirst(Path, faAnyFile, F) <> 0 then
    exit;
  Rows := FileTree.XML(Node).Add('<directory/>');
  { Folders }
  repeat
    if (F.Name <> '.') and (F.Name <> '..') and ((F.Attr and faHidden) = 0) then
    begin
      Row := Rows.Add('R');
      Row['NAME'] := F.Name;
      Row['PATH'] := ExtractFilePath(path);
      Row['EXT'] := ExtractFileExt(F.Name);
      {$IFDEF DXE2UP}
      Row.DateTime['TIME'] := F.TimeStamp;
      {$ENDIF}
      if (F.Attr and faDirectory) > 0 then
      begin
        Row['FILETYPE']:= 'DIRECTORY';
        Row['CHILDCOUNT'] := '1';
        Row['INDEX'] := '0' + AnsiLowerCase(F.Name);
        Ext := Row['PATH'] + F.Name + '\';
        if not ExtList.Find(Ext, n) then
        begin
          n := AddExtIcon(Ext, FILE_ATTRIBUTE_DIRECTORY);
          ExtList.AddObject(Ext, TStringRec.Create(LastFileType));
        end;
        Row['IMAGEINDEX'] := Inttostr(n)
      end else
      begin
        Ext := AnsiLowerCase(ExtractFileExt(F.Name));
        if StrIn(AnsiUpperCase(Ext), ['.RTF', '.DOC', '.DOCX', '.PPT', '.PPTX', '.XLS', '.XLSX', '.XLSB', '.EPUB', '.PDF',
         '.EMF', '.WMF', '.MSG', '.SVG', '.HTM', '.EML', '.FB2']) then
        begin
          Row['SIZE'] := inttostr(F.Size);
          Row['CHILDCOUNT'] := '0';
          Row['INDEX'] := '1' + AnsiLowerCase(F.Name);
          if Ext <> '' then
          begin
            if not ExtList.Find(Ext, n) then
            begin
              n := AddExtIcon(Ext, FILE_ATTRIBUTE_NORMAL);
              ExtList.AddObject(Ext, TStringRec.Create(LastFileType));
            end;
            Row['IMAGEINDEX'] := Inttostr(n)
          end;
        end else
          Rows.DeleteChild(Row);
      end;
    end;
    t := FindNext(F);
  until t <> 0;
  FindClose(F);
  Rows.SortbyAttr('INDEX');
  FileTree.LoadfromXML(Rows, Node);
end;

procedure TMainForm.LayoutModeChange(Sender: TObject);
begin
  FileTreeFocusChanged(FileTree, FileTree.FocusedNode, 0)
end;

procedure TMainForm.PrintBtnClick(Sender: TObject);
begin
  ShowHtPrintPreview(HtPanel1.Doc, HtPanel1.Styles.Text);
end;

function TMainForm.RequestPass(Sender: TObject; var Password: hstring): boolean;
begin
  Result := InputQuery('Enter password for ' + THtOfficeDocument(Sender).FileName, 'Password:', Password)
end;

procedure TMainForm.ToolButton1Click(Sender: TObject);
var D: TDOCXDocument;
    ST: TFileStream;
begin
  if SaveDialog2.Execute then
  begin
    D := TDOCXDocument.CreateNew;
    try
      D.LoadfromHTML(HtPanel1.Doc);
      ST := TFileStream.Create(SaveDialog2.FileName, fmCreate);
      D.SaveDOCX(ST);
      ST.Free
    finally
      D.Free
    end;
  end;
end;

procedure TMainForm.ViewInBrowserBtnClick(Sender: TObject);
var s: string;
begin
   s := ExtractFilePath(Paramstr(0)) + 'temp.html';
   TFile.WriteAllText(s, HtPanel1.Doc.OuterHTML, TEncoding.UTF8);
   ShellExecute(0, 'open', pchar(s), nil, nil, SW_SHOWNORMAL);
end;

{ TStringRec }

constructor TStringRec.Create(const Value: string);
begin
  s := Value
end;

end.
